/**
 * Copyright (c) 2009 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/

// modsDlg.cpp : implementation file
//

using System;
using System.Runtime.InteropServices;

namespace MyScreenGrabber
{
	/// <summary>
	/// This class shall keep the User32 APIs being used in 
	/// our program.
	/// </summary>
	public class USER32
	{

		#region Class Variables
		public  const int SM_CXSCREEN = 0;
		public  const int SM_CYSCREEN = 1;

        public const int MF_BYPOSITION = 0x400;
        public const int MF_DISABLED = 0x2;
        public const int SC_CLOSE = 0xF060;
        public const int MF_BYCOMMAND = 0x0;

		#endregion

        #region Class Structure
        [StructLayout(LayoutKind.Sequential)]
        public struct POINT
        {
            [MarshalAs(UnmanagedType.I4)]
            public int x;
            [MarshalAs(UnmanagedType.I4)]
            public int y;

            public POINT(int x, int y)
            {
                this.x = x;
                this.y = y;
            }

            public static implicit operator System.Drawing.Point(POINT p)
            {
                return new System.Drawing.Point(p.x, p.y);
            }

            public static implicit operator POINT(System.Drawing.Point p)
            {
                return new POINT(p.X, p.Y);
            }
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct RECT
        {
            [MarshalAs(UnmanagedType.I4)]
            public int left;
            [MarshalAs(UnmanagedType.I4)]
            public int top;
            [MarshalAs(UnmanagedType.I4)]
            public int right;
            [MarshalAs(UnmanagedType.I4)]
            public int bottom;

            public RECT(int l, int t, int r, int b)
            {
                this.left = l;
                this.top = t;
                this.right = r;
                this.bottom = b;
            }

            public static implicit operator System.Drawing.Rectangle(RECT r)
            {
                return new System.Drawing.Rectangle(r.left, r.top, r.right - r.left, r.bottom - r.top);
            }

            public static implicit operator RECT(System.Drawing.Rectangle r)
            {
                return new RECT(r.Left, r.Top, r.Right, r.Bottom);
            }
        }
        #endregion

        #region Windows Functions
        [DllImport("Coredll.dll", EntryPoint = "GetDesktopWindow")]
		public static extern IntPtr GetDesktopWindow();

        [DllImport("Coredll.dll", EntryPoint = "GetDC")]
		public static extern IntPtr GetDC(IntPtr ptr);

        [DllImport("Coredll.dll", EntryPoint = "GetSystemMetrics")]
		public static extern int GetSystemMetrics(int abc);

        [DllImport("Coredll.dll", EntryPoint = "GetWindowDC")]
		public static extern IntPtr GetWindowDC(Int32 ptr);

        [DllImport("Coredll.dll", EntryPoint = "ReleaseDC")]
		public static extern IntPtr ReleaseDC(IntPtr hWnd,IntPtr hDc);

        [DllImport("Coredll.dll", EntryPoint = "ClientToScreen")]
        public static extern bool ClientToScreen(IntPtr hWnd, ref POINT lpPoint);

        [DllImport("Coredll.dll", EntryPoint = "GetWindowRect")]
        public static extern bool GetWindowRect(IntPtr hWnd, ref RECT lpRect);

        [DllImport("Coredll.dll", EntryPoint = "GetClientRect")]
        public static extern bool GetClientRect(IntPtr hWnd, ref RECT lpRect);

        [DllImport("Coredll.dll", EntryPoint = "MoveWindow")]
        public static extern int MoveWindow(IntPtr hWnd, int X, int Y, int nWidth, int nHeight, bool bRepaint);
		
		#endregion

        #region Menu functions

        [DllImport("user32.dll", EntryPoint = "GetSystemMenu")]
        public static extern IntPtr GetSystemMenu(IntPtr hWnd, int revert);

        [DllImport("user32.dll", EntryPoint = "GetMenuItemCount")]
        public static extern int GetMenuItemCount(IntPtr hmenu);

        [DllImport("user32.dll", EntryPoint = "RemoveMenu")]
        public static extern int RemoveMenu(IntPtr hmenu, int npos, int wflags);

        [DllImport("user32.dll", EntryPoint = "DrawMenuBar")]
        public static extern int DrawMenuBar(IntPtr hmenu);

        #endregion


        #region Public Constructor
        public USER32()
		{
			// 
			// TODO: Add constructor logic here
			//
		}
		#endregion
	}
}
