/**
 * Copyright (c) 2009 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/

// modsDlg.cpp : implementation file
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MassWare;
using System.Net;
using System.IO;
using MyScreenGrabber;

namespace StudentPDA
{
    public partial class FormVideo : Form
    {
        private MassWare.MassWare m_massWare = null;
        //private MassWare.MassWare.AppDataReadyEventHandler DataReadyHandler;

        private String m_strTeacherName, m_strStudentName;

        private MySockets.TCPClient c_chbox;
        private MySockets.UDPServer c_slide;

        public delegate void UpdateTextCallback(string text);
        public delegate void UpdateVideoCallback(Image im);


        public FormVideo()
        {
            InitializeComponent();

            this.Text = "Video Window";

            m_massWare = new MassWare.MassWare("elRules.xml");
            m_massWare.Initialize();

            //DataReadyHandler = new MassWare.MassWare.AppDataReadyEventHandler(DataREadyCallBack);
            //MassWare.MassWare.AppDataReadyEvent += DataReadyHandler;

            object[] paramlist = { picBox, picBox.Width, picBox.Height };
            m_massWare.SetAppParameter("Masslets.Display.Display", "SetDisplayInf", paramlist);

            UpdateControls(false);
        }

        #region system event functions
        private void menuItemConnect_Click(object sender, EventArgs e)
        {
            FormConnect formConn = new FormConnect();
            if (formConn.ShowDialog() == DialogResult.OK)
            {
                m_strStudentName = formConn.mStrName;
                try
                {
                    // get local ip address
                    // Dns.GetHostEntry(connectForm.strIP).AddressList[0];
                    IPAddress remoteIPAddress = IPAddress.Parse(formConn.mStrIP);

                    /////////////start the chat box function
                    c_chbox = new MySockets.TCPClient(formConn.mStrIP, formConn.nPortChBox);
                    c_chbox.ReceivedMessage += new MySockets.TCPClient.ReceivedMsgEventHandler(c_chBox_RecievedMessage);
                    c_chbox.Start();

                    ////////////start the slides receiver
                    c_slide = new MySockets.UDPServer(MySockets.UDPServer.PORTSLIDE);
                    c_slide.ReceivedMessage += new MySockets.UDPServer.ReceivedMsgEventHandler(c_slide_ReceivedMessage);
                    //c_slide.Start();

                    if (c_chbox.Connected) UpdateControls(true);
                }
                catch (Exception se)
                {
                    MessageBox.Show("\nConnection failed, is the server running?\n" + se.Message);
                    UpdateControls(false);
                } 
            }
            formConn.Dispose();
        }

        private void txtBoxSend_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Return)
            {
                btnSend_Click(sender, e);
                e.Handled = true;
            }
        }

        private void btnSend_Click(object sender, EventArgs e)
        {
            if (txtBoxSend.Text == "") return;
            try
            {
                txtBoxSend.Text += "\r\n";
                byte[] byData = System.Text.Encoding.ASCII.GetBytes(txtBoxSend.Text);
                if (c_chbox != null) c_chbox.Send(byData);

                AppendText("You: " + txtBoxSend.Text);
                txtBoxSend.Text = "";
            }
            catch (Exception se)
            {
                MessageBox.Show(se.Message);
            }

        }

        private void menuItemDisc_Click(object sender, EventArgs e)
        {

        }

        private void menuItemStart_Click(object sender, EventArgs e)
        {
            //m_massWare.Start();
        }

        private void menuItemStop_Click(object sender, EventArgs e)
        {
            //m_massWare.Stop();
        }

        private void menuItemExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void FormVideo_Closing(object sender, CancelEventArgs e)
        {
            CloseSockets();
            m_massWare.Dispose();
            GC.Collect();
        }

        private void tabCtr_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tabCtr.SelectedIndex == 0)
            {
                if (c_slide != null) c_slide.Pause();
                if (m_massWare != null) m_massWare.PauseReceiving();
            }
            if (tabCtr.SelectedIndex == 1)
            {
                if (c_slide != null) c_slide.Resume();
                if (m_massWare != null) m_massWare.PauseReceiving();
            }
            if (tabCtr.SelectedIndex == 2)
            {
                if (c_slide != null) c_slide.Pause();
                if (m_massWare != null) m_massWare.ResumeReceiving();
            }
        }

        #endregion

        #region callback functions
        private void c_chBox_RecievedMessage(object sender, MySockets.TCPSocketEventArgs e)
        {
            String strDisp;

            switch (e.m_seq)
            {
                case -1: // connection closed forcely by remote server
                    {
                        menuItemDisc_Click(null, null);
                        return;
                    }
                case 0: // connected to the server, not message
                    {
                        // Send first massage, the name of the student
                        byte[] byData = System.Text.Encoding.ASCII.GetBytes(m_strStudentName);
                        c_chbox.Send(byData);
                        return;
                    }
                case 1: // received the first message
                    {
                        // get the name of the teacher
                        m_strTeacherName = Encoding.ASCII.GetString(e.m_msg, 0, e.m_msg.Length);

                        // Display this server connection as a status message on the GUI	
                        strDisp = String.Format("You have been connected to Teacher : {0}\r\n", m_strTeacherName);

                        break;
                    }
                default: // the rest message
                    {
                        strDisp = m_strTeacherName + ": " + Encoding.ASCII.GetString(e.m_msg, 0, e.m_msg.Length);
                        break;
                    }
            }

            // display to richtextbox
            txtBoxMemo.Invoke(new UpdateTextCallback(AppendText), new object[] { strDisp });

        }

        private void c_slide_ReceivedMessage(object sender, MySockets.UDPSocketEventArgs e)
        {
            MemoryStream ms = new MemoryStream(e.m_data);
            Image remoteImage = new Bitmap(ms);
            picBoxSlide.Invoke(new UpdateVideoCallback(DisplayREmoteSlide), new object[] { remoteImage });
        }


        private void DataREadyCallBack(object sender, MassWare.AppEventArgs e)
        {
            //use invoke to update picBox
            MemoryStream ms = new MemoryStream(e.mData);
            picBox.Image = new Bitmap(ms);
        }
        #endregion

        #region self defined functions
        private void UpdateControls(bool connected)
        {
            menuItemConnect.Enabled = !connected;
            menuItemDisc.Enabled = connected;
        }

        private void CloseSockets()
        {
            if (c_chbox != null) c_chbox.Close();
            if (c_slide != null) c_slide.Close();
        }

        #endregion

        int bmpseq = 0;
        private void menuItem1_Click(object sender, EventArgs e)
        {
            // save the screen shot
            Image im = CaptureWindow.GetScreenImage();
            string aName = System.Reflection.Assembly.GetExecutingAssembly().GetModules()[0].FullyQualifiedName;
            string path = System.IO.Path.GetDirectoryName(aName) + @"\";
            bmpseq++;
            im.Save(path + "a" + bmpseq + ".bmp", System.Drawing.Imaging.ImageFormat.Bmp);
        }

    }
}